@@ -1,5 +1,6 @@
 from enum import Enum
-from typing import Any, Dict, Tuple, Callable
+from typing import Any, Dict, Tuple, Callable, TypeVar, cast
+from robot.libraries.BuiltIn import BuiltIn  # type: ignore
 import re
 
 AssertionOperator = Enum(
@@ -27,6 +28,8 @@ AssertionOperator = Enum(
         "ends": "$=",
         "$=": "$=",
         "matches": "$",
+        "validate": "validate",
+        "then": "then",
     },
 )
 
@@ -49,13 +52,24 @@ handlers: Dict[AssertionOperator, Tuple[Callable, str]] = {
         lambda a, b: re.search(f"{re.escape(b)}$", a),
         "should end with",
     ),
+    AssertionOperator["validate"]: (
+        lambda a, b: BuiltIn().evaluate(b, namespace={"value": a}),
+        "should validate to true with",
+    ),
 }
 
+T = TypeVar("T")
 
-def verify_assertion(value: Any, operator: AssertionOperator, expected, message=""):
+
+def verify_assertion(
+    value: T, operator: AssertionOperator, expected: Any, message=""
+) -> T:
+    if operator is AssertionOperator["then"]:
+        return cast(T, BuiltIn().evaluate(expected, namespace={"value": value}))
     handler = handlers.get(operator)
     if handler is None:
         raise RuntimeError(f"{message} `{operator}` is not a valid assertion operator")
     validator, text = handler
     if not validator(value, expected):
         raise AssertionError(f"{message} `{value}` {text} `{expected}`")
+    return value
