@@ -2,5 +2,6 @@ from .browser_control import Control
 from .getters import Getters
 from .input import Input
 from .validation import Validation
+from .waiter import Waiter
 
-__all__ = ["Control", "Getters", "Input", "Validation"]
+__all__ = ["Control", "Getters", "Input", "Validation", "Waiter"]
