@@ -1,7 +1,14 @@
 import { IPlaywrightServer, PlaywrightService } from './generated/playwright_grpc_pb';
 import { chromium, firefox, webkit, Browser, BrowserContext, Page } from 'playwright';
 import { sendUnaryData, ServerUnaryCall, Server, ServerCredentials } from 'grpc';
-import { Response, Request } from './generated/playwright_pb';
+import { Response, Request, SelectEntry } from './generated/playwright_pb';
+
+declare global {
+    interface Window {
+        __SET_RFBROWSER__: <T>(a: T) => T;
+        __RFBROWSER__: any;
+    }
+}
 
 // This is necessary for improved typescript inference
 /*
@@ -14,7 +21,11 @@ function exists<T1, T2>(obj: T1, callback: sendUnaryData<T2>, message: string):
 }
 
 // Can't have an async constructor, this is a workaround
-async function createBrowserState(browserType: string, headless: boolean): Promise<BrowserState> {
+async function createBrowserState(
+    browserType: string,
+    headless: boolean,
+    hideRfBrowser: boolean,
+): Promise<BrowserState> {
     let browser;
     if (browserType === 'firefox') {
         browser = await firefox.launch({ headless: headless });
@@ -26,6 +37,14 @@ async function createBrowserState(browserType: string, headless: boolean): Promi
         throw new Error('unsupported browser');
     }
     const context = await browser.newContext();
+    if (!hideRfBrowser) {
+        context.addInitScript(function () {
+            window.__SET_RFBROWSER__ = function (state: any) {
+                window.__RFBROWSER__ = state;
+                return state;
+            };
+        });
+    }
     context.setDefaultTimeout(parseFloat(process.env.TIMEOUT || '10000'));
     const page = await context.newPage();
     return new BrowserState(browser, context, page);
@@ -68,12 +87,12 @@ class PlaywrightServer implements IPlaywrightServer {
         const url = call.request.getUrl();
         const headless = call.request.getHeadless();
         console.log('Open browser: ' + browserType);
-        this.browserState = await createBrowserState(browserType, headless);
+        this.browserState = await createBrowserState(browserType, headless, false);
         if (url) {
             await this.browserState.page.goto(url);
-            callback(null, emptyWithLog(`Succesfully opened browser ${browserType} to ${url}.`));
+            callback(null, emptyWithLog(`Successfully opened browser ${browserType} to ${url}.`));
         } else {
-            callback(null, emptyWithLog(`Succesfully opened browser ${browserType}.`));
+            callback(null, emptyWithLog(`Successfully opened browser ${browserType}.`));
         }
     }
 
@@ -140,10 +159,62 @@ class PlaywrightServer implements IPlaywrightServer {
     }
 
     // TODO: work some of getDomProperty and getBoolProperty's duplicate code into a root function
-    async getDomProperty(
-        call: ServerUnaryCall<Request.getDomProperty>,
-        callback: sendUnaryData<Response.String>,
+    async getSelectContent(
+        call: ServerUnaryCall<Request.selector>,
+        callback: sendUnaryData<Response.Select>,
     ): Promise<void> {
+        exists(this.browserState, callback, 'Tried to get Select element contents, no open browser');
+        const selector = call.request.getSelector();
+        const page = this.browserState.page;
+
+        type Value = [string, string, boolean];
+        const content: Value[] = await page.$$eval(selector + ' option', (elements) =>
+            (elements as HTMLOptionElement[]).map((elem) => [elem.label, elem.value, elem.selected]),
+        );
+
+        const response = new Response.Select();
+        content.forEach((option) => {
+            const [label, value, selected] = [option[0], option[1], option[2]];
+            const entry = new SelectEntry();
+            entry.setLabel(label);
+            entry.setValue(value);
+            entry.setSelected(selected);
+            response.addEntry(entry);
+        });
+        callback(null, response);
+    }
+
+    async selectOption(
+        call: ServerUnaryCall<Request.selectOption>,
+        callback: sendUnaryData<Response.Empty>,
+    ): Promise<void> {
+        exists(this.browserState, callback, 'Tried to select ``select`` element options, no open browser');
+        const selector = call.request.getSelector();
+        const matcher = JSON.parse(call.request.getMatcherjson());
+        console.log(`Selecting from element ${selector} options ${matcher}`);
+        const result = await this.browserState.page.selectOption(selector, matcher).catch((e) => {
+            callback(e, null);
+            throw e;
+        });
+
+        if (result.length == 0) {
+            console.log("Couldn't select any options");
+            const error = new Error(`No options matched ${matcher}`);
+            callback(error, null);
+        }
+        const response = emptyWithLog(`Selected options ${result} in element ${selector}`);
+        callback(null, response);
+    }
+
+    async deselectOption(call: ServerUnaryCall<Request.selector>, callback: sendUnaryData<Response.Empty>) {
+        exists(this.browserState, callback, 'Tried to deselect ``select`` element options, no open browser');
+        const selector = call.request.getSelector();
+        await this.browserState.page.selectOption(selector, []).catch((e) => callback(e, null));
+
+        callback(null, emptyWithLog(`Deselected options in element ${selector}`));
+    }
+
+    async _getDomProperty<T>(call: ServerUnaryCall<Request.getDomProperty>, callback: sendUnaryData<T>) {
         exists(this.browserState, callback, 'Tried to get DOM property, no open browser');
         const selector = call.request.getSelector();
         const property = call.request.getProperty();
@@ -160,7 +231,14 @@ class PlaywrightServer implements IPlaywrightServer {
         });
         const content = await result.jsonValue();
         console.log(`Retrieved dom property for element ${selector} containing ${content}`);
+        return content;
+    }
 
+    async getDomProperty(
+        call: ServerUnaryCall<Request.getDomProperty>,
+        callback: sendUnaryData<Response.String>,
+    ): Promise<void> {
+        const content = await this._getDomProperty(call, callback).catch((e) => callback(e, null));
         const response = new Response.String();
         response.setBody(content);
         callback(null, response);
@@ -170,23 +248,7 @@ class PlaywrightServer implements IPlaywrightServer {
         call: ServerUnaryCall<Request.getDomProperty>,
         callback: sendUnaryData<Response.Bool>,
     ): Promise<void> {
-        exists(this.browserState, callback, 'Tried to get DOM property, no open browser');
-        const selector = call.request.getSelector();
-        const property = call.request.getProperty();
-
-        const element = await this.browserState.page.$(selector).catch((e) => {
-            callback(e, null);
-            throw e;
-        });
-        exists(element, callback, "Couldn't find element: " + selector);
-
-        const result = await element.getProperty(property).catch((e) => {
-            callback(e, null);
-            throw e;
-        });
-        const content = await result.jsonValue();
-        console.log(`Retrieved dom property for element ${selector} containing ${content}`);
-
+        const content = await this._getDomProperty(call, callback).catch((e) => callback(e, null));
         const response = new Response.Bool();
         response.setBody(content || false);
         callback(null, response);
@@ -342,6 +404,46 @@ class PlaywrightServer implements IPlaywrightServer {
         const response = emptyWithLog('added Style: ' + content);
         callback(null, response);
     }
+
+    async waitForElementsState(
+        call: ServerUnaryCall<Request.selectorOptions>,
+        callback: sendUnaryData<Response.Empty>,
+    ): Promise<void> {
+        exists(this.browserState, callback, 'Tried to wait for an element, no open browser');
+        console.log('Waiting for element state');
+        const selector = call.request.getSelector();
+        const options = JSON.parse(call.request.getOptions());
+        await this.browserState.page.waitForSelector(selector, options).catch((e) => {
+            callback(e, null);
+            throw e;
+        });
+        const response = emptyWithLog('Wait for Element with selector: ' + selector);
+        callback(null, response);
+    }
+
+    async executeJavascriptOnPage(
+        call: ServerUnaryCall<Request.jsExecution>,
+        callback: sendUnaryData<Response.jsResult>,
+    ): Promise<void> {
+        exists(this.browserState, callback, 'Tried to execute js, no open browser');
+        const result = await this.browserState.page.evaluate(call.request.getScript());
+        const response = new Response.jsResult();
+        response.setLog('DUMMY');
+        response.setResult(JSON.stringify(result));
+        callback(null, response);
+    }
+
+    async getPageState(
+        call: ServerUnaryCall<Request.Empty>,
+        callback: sendUnaryData<Response.jsResult>,
+    ): Promise<void> {
+        exists(this.browserState, callback, 'Tried to get page state, no open browser');
+        const result = await this.browserState.page.evaluate(() => window.__RFBROWSER__);
+        console.log(result);
+        const response = new Response.jsResult();
+        response.setResult(JSON.stringify(result));
+        callback(null, response);
+    }
 }
 
 const server = new Server();
